package excel_import

import (
	"github.com/xuri/excelize/v2"
	"io/fs"
	"time"
)

type (
	Task struct {
		TaskId  string `json:"task_id"`
		Process int    `json:"process"`
		Ctime   string `json:"ctime"`
		Ftime   string `json:"ftime"`
		Status  int    `json:"status"`
	}

	ImportLog struct {
		FailReason string `json:"fail_reason"`
		Line       int    `json:"line"`
	}

	RowInfo struct {
		Row []string
	}

	TaskResp struct {
		Task         *Task        `json:"task"`
		ImportErrLog []*ImportLog `json:"log"`
	}

	FileInfoStatus struct {
		fs.FileInfo
		Status int
		Time   time.Time
	}

	ResPage struct {
		Page     int                      `json:"current_page"`
		Limit    int                      `json:"per_page"`
		Total    int                      `json:"total"`
		LastPage int                      `json:"last_page"`
		Data     []map[string]interface{} `json:"data"`
	}

	PicList struct {
		PicInfos []excelize.Picture
		Url      []string
	}

	Pic struct {
		PicInfos excelize.Picture
		Url      string
		Cell     string
	}

	//Header struct {
	//	HeaderMap map[string]string
	//	SortSlice []string
	//}
)

const (
	TaskStatusInit = iota + 1 // 初始化
	TaskStatusRunning
	TaskStatusCreateFailExcel
	TaskStatusFinish
)

const (
	Rows_From_Request = iota + 1
	Rows_From_File
	Rows_From_Rows
)

var fromWays = map[int]string{
	Rows_From_Request: "request",
	Rows_From_File:    "file",
	Rows_From_Rows:    "rows",
}
